import 'dart:io';

import 'package:dct/dct.dart' as dct;

import 'package:cli_util/cli_logging.dart';
import 'package:cli_util/cli_util.dart';

import 'package:args/args.dart';
import 'package:path/path.dart' as path;
import 'package:platform_info/platform_info.dart';

void main(List<String> args) {
  final dct_version = '0.0.3';
  final _platform = Platform();
  final sdk_path = getSdkPath();

  final is_windows = _platform.version.contains('Windows');

  final dart2native_path = is_windows
      ? path.normalize(sdk_path + '/bin/dart2native.bat')
      : path.normalize(sdk_path + '/bin/dart2native');

  const switch_input_expectation = {
    '--version': false,
    '-V': false,
    '--debug': false,
    '-d': false,
    '--use-version': true,
    '-v': true,
    '--lock': true,
    '-l': true,
    '--remove': true,
    '-r': true,
    '--help': false,
    '-h': false
  };

  String package_name;
  List<String> dct_args = [];
  List<String> package_args;

  // iterate over the arguments
  for (var i = 0; i < args.length; i++) {
    if (args[i].startsWith('-')) {
      // it's an argument, so add it to dct_args
      dct_args.add(args[i]);
    } else {
      // first, check if the last arg processed expects input
      var last_arg_expects_input;

      if (dct_args.isEmpty) {
        // if the args array is empty, the "last" arg isn't expecting input
        last_arg_expects_input = false;
      } else {
        last_arg_expects_input = switch_input_expectation[dct_args.last];
      }

      // (catch if it's null)
      last_arg_expects_input ??= false;

      // if the last arg is expecting input add the current item to dct_args
      if (last_arg_expects_input) dct_args.add(args[i]);

      // if not, we've just found our package name, so..
      if (!last_arg_expects_input) {
        // ..set the package_name variable..
        package_name = args[i];
        // ..capture the package's args..
        package_args = args.sublist(i + 1, args.length);
        // ..and stop the loop
        break;
      }
    }
  }

  var parser = ArgParser()
    ..addFlag('version',
        abbr: 'V',
        negatable: false,
        defaultsTo: false,
        help: 'Print the current dct version')
    ..addFlag('debug',
        abbr: 'd',
        negatable: false,
        defaultsTo: false,
        help: 'Run in debug mode')
    ..addOption('use-version',
        abbr: 'v', help: 'Run the specified version of a package')
    ..addOption('lock',
        abbr: 'l', help: 'Run and lock a package at the specified version')
    ..addFlag('remove',
        abbr: 'r',
        negatable: false,
        defaultsTo: false,
        help: 'Remove the package from your computer (Can be coupled with -v)')
    ..addFlag('help', abbr: 'h', help: 'Show this help text', negatable: false);

  var arguments = parser.parse(dct_args);

  var logger = arguments['debug'] ? Logger.verbose() : Logger.standard();
  logger.trace('dct version $dct_version on ${_platform.version}');
  logger.trace('SDK path is $sdk_path');
  logger.trace(
      'SDK version is ${File(path.join(sdk_path, 'version')).readAsStringSync()}');
  logger.trace('dart2native path is $dart2native_path');

  logger.trace('Arguments for dct: $dct_args');
  logger.trace('Package name: $package_name');
  logger.trace('Arguments for $package_name: $package_args');

  if (arguments['help']) {
    logger.trace('help argument specified, outputting help info');
    stdout.writeln('Usage:\n\n${parser.usage}');
    exit(0);
  }

  // if the package name isn't defined, quit
  if (package_name == null) {
    logger.stderr('No package defined');
    exit(2);
  }
}
